/// <reference types="express" />
import * as express from 'express';
import { StaticModelT, ModelT } from '../core/model';
import { CountAllResults } from '../core/db';
export declare type PerformQueryT = {
    where: Object;
    include: string[];
    order: [string, string][];
    offset: number;
    limit: number;
};
export declare abstract class CrudController<T extends ModelT<any>> {
    private staticModel;
    protected readonly modelName: string;
    constructor(staticModel: StaticModelT<T>, modelName: string, pluralName?: string, singularName?: string);
    private splitOnWords(name);
    private getSingularPath(name);
    private getPluralPath(name);
    private singularName;
    private pluralName;
    transformPathPart(part: string): string;
    protected transformQuery(req: express.Request, res: express.Response, query: Object): Promise<Object>;
    protected performQuery(req: express.Request, res: express.Response, query: PerformQueryT): Promise<CountAllResults<T>>;
    protected transformQueryResults(req: express.Request, res: express.Response, results: CountAllResults<T>): Promise<{
        results: T[];
        count: number;
    } | undefined>;
    protected transformResult(req: express.Request, res: express.Response, result: T | null): Promise<T | null>;
    protected transformCreateQuery(req: express.Request, res: express.Response, query: Object): Promise<Object>;
    protected transformCreateResult(req: express.Request, res: express.Response, result: T): Promise<T>;
    protected transformUpdateQuery(req: express.Request, res: express.Response, query: Object): Promise<Object>;
    protected transformUpdateResult(req: express.Request, res: express.Response, result: T): Promise<T>;
    create(req: express.Request, res: express.Response): Promise<void>;
    protected afterCreate(req: express.Request, res: express.Response, result: T): Promise<void>;
    find(req: express.Request, res: express.Response): Promise<void>;
    count(req: express.Request, res: express.Response): Promise<void>;
    get(req: express.Request, res: express.Response): Promise<void>;
    update(req: express.Request, res: express.Response): Promise<void>;
    destroy(req: express.Request, res: express.Response): Promise<void>;
}
