"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const routes_1 = require("../decorators/router/routes");
const pluralize_1 = require("../util/pluralize");
const http_status_type_1 = require("../util/http-status-type");
class CrudController {
    constructor(staticModel, modelName, pluralName, singularName) {
        this.staticModel = staticModel;
        this.modelName = modelName;
        this.singularName = singularName || this.getSingularPath(modelName);
        this.pluralName = pluralName || this.getPluralPath(modelName);
    }
    *splitOnWords(name) {
        let currentWord = '';
        for (let q = 0; q < name.length; q++) {
            let chr = name[q];
            if (chr.match(/[A-Z]/)) {
                if (currentWord)
                    yield currentWord;
                currentWord = chr;
            }
            else if (chr == '_') {
                if (currentWord)
                    yield currentWord;
                currentWord = '';
            }
            else
                currentWord += chr;
        }
        if (currentWord)
            yield currentWord;
    }
    getSingularPath(name) {
        let parts = [...this.splitOnWords(name)].filter(Boolean);
        if (!parts.length)
            return name;
        parts[parts.length - 1] = pluralize_1.pluralize(parts[parts.length - 1], false);
        return parts.map(pt => pt.toLowerCase()).join('-');
    }
    getPluralPath(name) {
        let parts = [...this.splitOnWords(name)].filter(Boolean);
        if (!parts.length)
            return name;
        parts[parts.length - 1] = pluralize_1.pluralize(parts[parts.length - 1]);
        return parts.map(pt => pt.toLowerCase()).join('-');
    }
    transformPathPart(part) {
        return part.replace(/%%PLURAL_NAME%%/, this.pluralName).replace(/%%SINGULAR_NAME%%/, this.singularName);
    }
    transformQuery(req, res, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return query;
        });
    }
    performQuery(req, res, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.staticModel.db.findAndCountAll(query);
        });
    }
    transformQueryResults(req, res, results) {
        return __awaiter(this, void 0, void 0, function* () {
            let initialStatusCode = res.statusCode;
            let oldResults = results.results;
            let newResults = [];
            for (let q = 0; q < oldResults.length; q++) {
                let transformed = yield this.transformResult(req, res, oldResults[q]);
                if (res.statusCode !== initialStatusCode || res.headersSent)
                    return;
                if (transformed)
                    newResults.push(transformed);
            }
            return {
                results: newResults,
                count: results.count
            };
        });
    }
    transformResult(req, res, result) {
        return __awaiter(this, void 0, void 0, function* () {
            return result;
        });
    }
    transformCreateQuery(req, res, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return query;
        });
    }
    transformCreateResult(req, res, result) {
        return __awaiter(this, void 0, void 0, function* () {
            return result;
        });
    }
    transformUpdateQuery(req, res, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return query;
        });
    }
    transformUpdateResult(req, res, result) {
        return __awaiter(this, void 0, void 0, function* () {
            return result;
        });
    }
    create(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            let initialStatusCode = res.statusCode;
            let data = yield this.transformCreateQuery(req, res, req.body);
            if (res.statusCode !== initialStatusCode || res.headersSent)
                return;
            if (!data) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`You haven't sent any data to create the ${this.modelName} with!`);
                return;
            }
            if (data.constructor == Array) {
                throw new Error("createMany not supported");
            }
            let result = yield this.staticModel.db.create(data);
            result = yield this.transformCreateResult(req, res, result);
            if (res.statusCode !== initialStatusCode || res.headersSent)
                return;
            this.afterCreate(req, res, result);
            if (res.statusCode !== initialStatusCode || res.headersSent)
                return;
            res.status(http_status_type_1.HTTP_STATUS_OK).json(result);
        });
    }
    afterCreate(req, res, result) {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    find(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            let query = {};
            let include = [];
            let order = [];
            try {
                if (req.query['query'])
                    query = JSON.parse(req.query['query'] || '{}');
                if (req.query['include'])
                    include = JSON.parse(req.query['include'] || '[]');
                if (req.query['order'])
                    order = JSON.parse(req.query['order'] || '[]');
            }
            catch (e) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Could not parse request parameters.`);
                return;
            }
            if ((!include || !include.length) && query.include) {
                include = query.include;
                delete query.include;
            }
            if ((!order || !order.length) && query.order) {
                order = query.order;
                delete query.order;
            }
            let initialStatusCode = res.statusCode;
            query = (yield this.transformQuery(req, res, query)) || query;
            if (res.statusCode !== initialStatusCode || res.headersSent)
                return;
            let perPage = req.query['perPage'];
            if (!perPage || !(perPage = parseInt('' + perPage, 10)) || isNaN(perPage) || perPage <= 0)
                perPage = 10;
            let page = req.query['page'];
            if (!page || !(page = parseInt('' + page, 10)) || isNaN(page) || page < 0)
                page = 0;
            let results = yield this.performQuery(req, res, {
                where: query,
                include: include,
                order: order,
                offset: page * perPage,
                limit: perPage
            });
            if (res.statusCode !== initialStatusCode || res.headersSent)
                return;
            results = (yield this.transformQueryResults(req, res, results));
            if (res.statusCode !== initialStatusCode || res.headersSent || !results)
                return;
            res.status(http_status_type_1.HTTP_STATUS_OK).json({
                results: results.results,
                page: page,
                perPage: perPage,
                total: results.count
            });
        });
    }
    count(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            let query = {};
            try {
                if (req.query['query'])
                    query = JSON.parse(req.query['query'] || '{}');
            }
            catch (e) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Could not parse query parameters`);
                return;
            }
            let initialStatusCode = res.statusCode;
            query = (yield this.transformQuery(req, res, query)) || query;
            if (res.statusCode !== initialStatusCode || res.headersSent)
                return;
            let count = yield this.staticModel.db.count({
                where: query
            });
            res.status(http_status_type_1.HTTP_STATUS_OK).send(`${count}`);
        });
    }
    get(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            let id = parseInt(req.params['id'], 10);
            if (!id || isNaN(id)) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Invalid ${this.modelName} id: ${req.params['id']}`);
                return;
            }
            let include = [];
            try {
                if (req.query['include'])
                    include = JSON.parse(req.query['include'] || '[]');
            }
            catch (e) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Could not parse request parameters.`);
                return;
            }
            let result = yield this.staticModel.db.findById(id, { include: include });
            let initialStatusCode = res.statusCode;
            result = yield this.transformResult(req, res, result);
            if (res.statusCode === initialStatusCode && !res.headersSent)
                res.status(http_status_type_1.HTTP_STATUS_OK).json(result);
        });
    }
    update(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            let id = parseInt(req.params['id'], 10);
            if (!id || isNaN(id)) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Invalid ${this.modelName} id: ${req.params['id']}`);
                return;
            }
            let initialStatusCode = res.statusCode;
            let data = yield this.transformUpdateQuery(req, res, req.body);
            if (res.statusCode !== initialStatusCode || res.headersSent) {
                return;
            }
            if (!data) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`You haven't sent any data to update the ${this.modelName} with!`);
                return;
            }
            let returning = false;
            let returningParam = req.query['returning'];
            if (returningParam) {
                if (returningParam == true || returningParam == 'true')
                    returning = true;
                else if (returningParam == false || returningParam == 'false')
                    returning = false;
                else {
                    res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Invalid ${this.modelName} returning parameter: ${returningParam}; must be boolean`);
                    return;
                }
            }
            let [updated, results] = yield this.staticModel.db.update(id, data, returning);
            initialStatusCode = res.statusCode;
            results = yield Promise.all(results.map((result) => this.transformUpdateResult(req, res, result)));
            if (res.statusCode !== initialStatusCode || res.headersSent) {
                return;
            }
            if (!updated) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Can't find the ${this.modelName} with id ${id} to update it.`);
                return;
            }
            if (returning)
                res.status(http_status_type_1.HTTP_STATUS_OK).json(results[0]);
            else
                res.status(http_status_type_1.HTTP_STATUS_OK).end();
        });
    }
    destroy(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            let id = parseInt(req.params['id'], 10);
            if (!id || isNaN(id)) {
                res.status(http_status_type_1.HTTP_STATUS_ERROR).send(`Invalid ${this.modelName} id: ${req.params['id']}`);
                return;
            }
            let destroyed = yield this.staticModel.db.destroy(id);
            res.status(http_status_type_1.HTTP_STATUS_OK).json({ destroyed: destroyed });
        });
    }
}
__decorate([
    routes_1.Post(`/%%PLURAL_NAME%%/create`),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], CrudController.prototype, "create", null);
__decorate([
    routes_1.Get(`/%%PLURAL_NAME%%/find`),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], CrudController.prototype, "find", null);
__decorate([
    routes_1.Get(`/%%PLURAL_NAME%%/count`),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], CrudController.prototype, "count", null);
__decorate([
    routes_1.Get(`/%%SINGULAR_NAME%%/:id`),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], CrudController.prototype, "get", null);
__decorate([
    routes_1.Put(`/%%SINGULAR_NAME%%/:id`),
    routes_1.Patch(`/%%SINGULAR_NAME%%/:id`),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], CrudController.prototype, "update", null);
__decorate([
    routes_1.Delete(`/%%SINGULAR_NAME%%/:id`),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], CrudController.prototype, "destroy", null);
exports.CrudController = CrudController;
//# sourceMappingURL=crud-controller.js.map