"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const expressJwt = require("express-jwt");
const policy_decorator_1 = require("../decorators/policies/policy.decorator");
const server_1 = require("../server/server");
const wrap_promise_1 = require("../util/wrap-promise");
let JwtBasePolicy = class JwtBasePolicy {
    constructor(server, credentialsRequired) {
        this.server = server;
        let jwt = server.meta.jwt;
        if (typeof jwt === 'undefined')
            return;
        this.property = jwt.tokenProperty || 'jwt';
        this.jwtHandler = expressJwt({
            secret: jwt.secret,
            userProperty: this.property,
            credentialsRequired: credentialsRequired
        });
    }
    get logger() {
        return this.server.logger;
    }
    handle(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.jwtHandler) {
                try {
                    yield wrap_promise_1.wrapPromise(this.jwtHandler, req, res);
                }
                catch (e) {
                    this.logger.verbose('jwt-policy', `express-jwt failed to parse 'Authorization' header.`);
                    this.logger.verbose('jwt-policy', `'Authorization' header: '${req.header('Authorization')}'`);
                    req[this.property] = undefined;
                }
            }
            if (req[this.property])
                return req[this.property] = yield this.fromJson(req[this.property]);
            return null;
        });
    }
    fromJson(json) {
        return __awaiter(this, void 0, void 0, function* () {
            return json;
        });
    }
};
JwtBasePolicy = __decorate([
    policy_decorator_1.Policy(),
    __metadata("design:paramtypes", [server_1.Server, Boolean])
], JwtBasePolicy);
exports.JwtBasePolicy = JwtBasePolicy;
//# sourceMappingURL=jwt-base.policy.js.map