"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const clc_1 = require("../util/clc");
const fs = require("fs");
const path = require("path");
class Logger {
    constructor(serverName, logLevel) {
        this._serverName = serverName;
        if (typeof logLevel === 'undefined')
            logLevel = { default: 'info' };
        if (typeof logLevel === 'string')
            logLevel = { default: logLevel };
        this.logLevel = logLevel;
        let logPath = !!serverName ? `log/${serverName}` : `log`;
        try {
            logPath.split('/').forEach((dir, index, splits) => {
                let partialPath = path.resolve(splits.splice(0, index).join('/'), dir);
                if (!fs.existsSync(partialPath))
                    fs.mkdirSync(partialPath);
            });
        }
        catch (e) {
            console.error(e);
        }
        this.fd = fs.openSync(`${logPath}/${serverName}.${new Date().getTime()}.log`, 'w');
    }
    ShutDown() {
        fs.close(this.fd);
    }
    get serverName() {
        return this._serverName;
    }
    logLabelMessage(subsystem) {
        if (!this.serverName && !subsystem)
            return '';
        else if (!this.serverName)
            return clc_1.clc.white(`[${subsystem}]`);
        else if (!subsystem)
            return clc_1.clc.white(`[${clc_1.clc.bold(clc_1.clc.yellow(this.serverName))}]`);
        else
            return clc_1.clc.white(`[${clc_1.clc.bold(clc_1.clc.yellow(this.serverName))}:${subsystem}]`);
    }
    logLevelCheck(subsystem, logLevel) {
        let allowedLevel = (subsystem && this.logLevel[subsystem]) || this.logLevel['default'];
        switch (allowedLevel) {
            case 'verbose':
                return true;
            case 'info':
                return logLevel !== 'verbose';
            case 'warn':
                return logLevel === 'warn' || logLevel === 'error';
            case 'error':
                return logLevel === 'error';
        }
    }
    writeToFile(message) {
        fs.write(this.fd, message + "\n", () => { });
    }
    log(subsystem, message, ...optionalParams) {
        this.writeToFile(`[${new Date().toISOString()}:${subsystem}]: ${message}`);
        console.log(this.logLabelMessage(subsystem), message, ...optionalParams);
    }
    trace(subsystem, message, ...optionalParams) {
        this.writeToFile(`[${new Date().toISOString()}:${subsystem}]: ${message}`);
        console.trace(this.logLabelMessage(subsystem), message, ...optionalParams);
    }
    error(subsystem, message, ...optionalParams) {
        this.writeToFile(`[${new Date().toISOString()}:${subsystem}] error: ${message}`);
        console.error(this.logLabelMessage(subsystem), clc_1.clc.error(`error:`), message, ...optionalParams);
    }
    info(subsystem, message, ...optionalParams) {
        this.writeToFile(`[${new Date().toISOString()}:${subsystem}] info: ${message}`);
        if (!this.logLevelCheck(subsystem, 'info'))
            return;
        console.info(this.logLabelMessage(subsystem), clc_1.clc.info(`info:`), message, ...optionalParams);
    }
    warn(subsystem, message, ...optionalParams) {
        this.writeToFile(`[${new Date().toISOString()}:${subsystem}] warn: ${message}`);
        if (!this.logLevelCheck(subsystem, 'warn'))
            return;
        console.warn(this.logLabelMessage(subsystem), clc_1.clc.warn(`warn:`), message, ...optionalParams);
    }
    verbose(subsystem, message, ...optionalParams) {
        this.writeToFile(`[${new Date().toISOString()}:${subsystem}] verbose: ${message}`);
        if (!this.logLevelCheck(subsystem, 'verbose'))
            return;
        console.log(this.logLabelMessage(subsystem), 'verbose:', message, ...optionalParams);
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map