"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class ServiceReflector {
    constructor(server) {
        this.server = server;
        this._startedServices = [];
    }
    get logger() {
        return this.server.logger;
    }
    reflectServices(services) {
        return __awaiter(this, void 0, void 0, function* () {
            let failures = 0;
            for (let q = 0; q < services.length; q++) {
                let result;
                try {
                    result = yield this.reflectService(services[q]);
                }
                catch (e) {
                    this.logger.error('services', `Exception occurred when trying to start service: ${services[q].name || services[q]}`);
                    this.logger.error('services', e);
                    result = false;
                }
                if (!result) {
                    this.logger.error('services', `Failed to start service: ${services[q].name || services[q]}`);
                    failures++;
                }
            }
            if (!!failures)
                this.logger.error('services', `${services.length - failures} services started correctly out of ${services.length}`);
            else
                this.logger.info('services', `${services.length - failures} services started correctly out of ${services.length}`);
        });
    }
    reflectService(serviceFn) {
        return __awaiter(this, void 0, void 0, function* () {
            let service = this.server.injector.resolveInjectable(serviceFn);
            if (typeof service === 'undefined')
                throw new Error(`Failed to inject service: ${serviceFn.name || serviceFn}`);
            this._startedServices.push(service);
            if (typeof service.start !== 'undefined') {
                let result = yield service.start();
                if (typeof result === 'boolean' && !result)
                    return false;
            }
            return true;
        });
    }
    shutdownServices() {
        return __awaiter(this, void 0, void 0, function* () {
            let services = this._startedServices;
            this._startedServices = [];
            let failures = 0;
            for (let q = 0; q < services.length; q++) {
                let result;
                try {
                    result = yield this.shutdownService(services[q]);
                }
                catch (e) {
                    this.logger.error('services', `Exception occurred when trying to stop service: ${services[q]}`);
                    this.logger.error('services', e);
                    result = false;
                }
                if (!result) {
                    this.logger.error('services', `Failed to start service: ${services[q]}`);
                    failures++;
                }
            }
            if (!!failures)
                this.logger.error('services', `${services.length - failures} services terminated correctly out of ${services.length}`);
            else
                this.logger.info('services', `${services.length - failures} services terminated correctly out of ${services.length}`);
        });
    }
    shutdownService(service) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof service.stop !== 'undefined') {
                let result = yield service.stop();
                if (typeof result === 'boolean' && !result)
                    return false;
            }
            return true;
        });
    }
}
exports.ServiceReflector = ServiceReflector;
//# sourceMappingURL=reflector.js.map